"use strict";
// Copyright IBM Corp. 2018. All Rights Reserved.
// Node module: @loopback/example-rpc-server
// This file is licensed under the MIT License.
// License text available at https://opensource.org/licenses/MIT
Object.defineProperty(exports, "__esModule", { value: true });
const rpc_server_1 = require("../../src/rpc.server");
const rpc_router_1 = require("../../src/rpc.router");
const testlab_1 = require("@loopback/testlab");
describe('rpcRouter', () => {
    describe('routeHandler', () => {
        // tslint:disable-next-line:no-any
        let server;
        let request;
        let response;
        let responseStub;
        beforeEach(testSetup);
        it('routes correctly when controller and method exist', async () => {
            await rpc_router_1.routeHandler(server, request, response);
            testlab_1.sinon.assert.called(responseStub);
            testlab_1.expect(responseStub.firstCall.args[0]).to.match(/Hello, Janet!/);
        });
        it('throws 400 when controller does not exist', async () => {
            const getStub = server.get;
            getStub.rejects(new Error('Does not exist!'));
            await rpc_router_1.routeHandler(server, request, response);
            testlab_1.expect(response.statusCode).to.equal(400);
            testlab_1.sinon.assert.called(responseStub);
            testlab_1.expect(responseStub.firstCall.args[0]).to.match(/Does not exist!/);
        });
        it('throws 400 when method does not exist', async () => {
            request = getRequest({
                body: {
                    controller: 'FakeController',
                    method: 'notReal',
                    input: {
                        name: 'Sad',
                    },
                },
            });
            await rpc_router_1.routeHandler(server, request, response);
            testlab_1.expect(response.statusCode).to.equal(400);
            testlab_1.sinon.assert.called(responseStub);
            testlab_1.expect(responseStub.firstCall.args[0]).to.match(/No method was found on controller/);
        });
        it('throws 500 on unhandled error', async () => {
            server.get.resolves(new class extends FakeController {
                // tslint:disable-next-line:no-any
                getFoo(input) {
                    throw new Error('>:(');
                }
            }());
            await rpc_router_1.routeHandler(server, request, response);
            testlab_1.expect(response.statusCode).to.equal(500);
            testlab_1.sinon.assert.called(responseStub);
            testlab_1.expect(responseStub.firstCall.args[0].message).to.match('>:(');
        });
        function testSetup() {
            server = getServer();
            request = getRequest();
            response = getResponse();
            responseStub = response.send;
        }
    });
    function getServer() {
        const server = testlab_1.sinon.createStubInstance(rpc_server_1.RPCServer);
        server.get.resolves(FakeController);
        return server;
    }
    function getRequest(req) {
        return Object.assign({
            body: {
                controller: 'FakeController',
                method: 'getFoo',
                input: {
                    name: 'Janet',
                },
            },
        }, req);
    }
    function getResponse(res) {
        const resp = {};
        resp.send = testlab_1.sinon.stub();
        return resp;
    }
    class FakeController {
        // tslint:disable-next-line:no-any
        getFoo(input) {
            return `Hello, ${input.name}!`;
        }
    }
});
//# sourceMappingURL=rpc.router.unit.js.map